import 'package:enough_mail/enough_mail.dart';

class MessageSequence {
  final List<int> _sequence = <int>[];
  String _text;
  bool _addLast = false;
  final Map<int, bool> _idsWithRangeToLast = <int, bool>{};
  final Map<int, int> _idsWithRange = <int, int>{};

  void addSequenceId(MimeMessage message) {
    add(message.sequenceId);
  }

  void addUid(MimeMessage message) {
    add(message.uid);
  }

  void add(int id) {
    _sequence.add(id);
    _text = null;
  }

  void addRange(int start, int end) {
    for (var i = start; i <= end; i++) {
      _sequence.add(i);
    }
    _text = null;
  }

  void addRangeToEnd(int start) {
    // problem - when adding several things such as "1, 17, 20:*, 15" then I lose track of where the last element belongs to
    // map with <int, bool>{} ?
    var sequenceWasEmpty = _sequence.isEmpty;
    _sequence.add(start);
    _idsWithRangeToLast[start] = true;
    if (sequenceWasEmpty) {
      _text = '$start:*';
    } else {
      _text = null;
    }
  }

  void addLast() {
    // adding *
    _addLast = true;
  }

  void addAll() {
    // 1:*
  }

  bool isEmpty() {
    return _sequence.isEmpty &&
        _idsWithRangeToLast.isEmpty &&
        _idsWithRange.isEmpty;
  }

  bool isNotEmpty() {
    return _sequence.isNotEmpty ||
        _idsWithRangeToLast.isNotEmpty ||
        _idsWithRange.isNotEmpty;
  }

  @override
  String toString() {
    if (_text != null) {
      return _text;
    }
    var buffer = StringBuffer();
    render(buffer);
    return buffer.toString();
  }

  void render(StringBuffer buffer) {
    if (_text != null) {
      buffer.write(_text);
      return;
    }
    if (_sequence.isEmpty) {
      if (_addLast) {
        buffer.write('*');
        return;
      }
      throw StateError('no ID added to sequence');
    }
    if (_sequence.length == 1) {
      buffer.write(_sequence[0]);
    } else {
      _sequence.sort();
      var doWrite = true;
      int last;
      for (var i = 0; i < _sequence.length; i++) {
        var current = _sequence[i];
        if (doWrite) {
          buffer.write(current);
        }
        last = current;
      }
    }
    if (_addLast) {
      buffer.write(',*');
    }
  }
}
